<?php
namespace Controller;

use Doctrine\ORM\EntityManager;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class UnidadeController {
    protected $dataSourceNames = array(
        'default'      => '',
        'filial'       => 'filial_view',
        'endefili'     => 'endefili_view',
        'almoxarifado' => 'almoxarifado',
        'localestoque' => 'localestoque',
        'paramfilial'  => 'paramfilial',
        'filipadr'     => 'paramfilipadr',
        'filioper'     => 'filioper'
    );
    
    protected $dataSourceNameToSave;
    protected $entityManager;
    protected $service;
    protected $crudService;
    private $environment;

    /**
     * Construtor do Controller.
     *
     * @param CrudService $crudService
     * @param EntityManager $entityManager
     * @param UnidadeService $service
     */
    public function __construct($crudService, EntityManager $entityManager, $environment, $service) {
        $this->entityManager = $this->entityManager? $this->entityManager : $entityManager;
        $this->crudService   = $crudService;
        $this->environment   = $environment;
        $this->service       = $service;

        $this->switchDataSource('default');
    }

    /**
     * Procedimento que recebe um request e realiza a
     * inserção ou alteração dos dados da unidade.
     *
     * Envia no objeto response um DataSet com uma
     * mensagem de erro caso necessário.
     *
     * Efetua rollback no caso de algum erro.
     *
     * ATENÇÃO! Esse método pressupõe que apenas uma row
     * será recebida! WARNING! AVVERTIMENCO! ADVERTENCIA!
     *
     * @param Filter $request
     * @param Response $response
     */
    public function saveFilial(DTO\Request $request, DTO\Response $response) {
        $this->switchDataSource('filial');
        $rows =  $request->getDataSet()->getRows();
        foreach($rows as $row) {
            $row = $this->service->validateUnidadeRow($row);
            
            $row['NRORG'] = $this->environment->getNrOrgTrab();
            
            // from here it's using temporary values
            /*$currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $row['DTULTATU'] = $currentTime;
            $row['NRORGULTATU'] = '1';
            $row['CDOPERULTATU'] = '000000078954';
            $row['DTINCLUSAO'] = $currentTime;
            $row['NRORGINCLUSAO'] = '1';
            $row['CDOPERINCLUSAO'] = '000000078954';
            $row['NRORG'] = '1';*/
            // temporary values ends here
        }
        $this->entityManager->beginTransaction();
        try {
            $this->saveGeneric($request, $response, 'filial', true);

            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new']) {
                    $row = $this->service->validateUnidadeNewRow($row);
                    
                    $this->saveGeneric($request, $response, 'almoxarifado', true);
                    $this->saveGeneric($request, $response, 'localestoque', true);
                    $this->saveGeneric($request, $response, 'paramfilial', true);

                    $NMTABS = array(
                        'ALMOXARIFADO', 'LOCALESTOQUE', 'PARAMFILIAL',
                        'PARAMPEDIDO', 'PARAMPLANEJA', 'PARCMPSUP', 'PAREXPEDICAO'
                    );

                    foreach ($NMTABS as $NMTABPARAMFILI) {
                        $row['NMTABPARAMFILI'] = $NMTABPARAMFILI;
                        $this->saveGeneric($request, $response, 'filipadr', true);
                    }
                    $this->saveEndefili($request, $response);
                }
            }

            $this->entityManager->commit();
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
            $this->entityManager->rollback();
        }

        $this->switchDataSource('default');
    }
    
    /**
     * Procedimento que recebe um request e realiza a
     * inserção ou alteração dos dados do endereço da
     * unidade.
     *
     * Envia no objeto response um DataSet com uma
     * mensagem de erro caso necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function saveEndefili(DTO\Request $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                $row = $this->service->validateEndefiliRow($row);
                if(isset($row['CDSENHEMAILF']) && !empty($row['CDSENHEMAILF'])){
                   $row['CDSENHEMAILF'] = $this->service->encrypt($row['CDSENHEMAILF'],2);
                }
            }
    
            $this->saveGeneric($request, $response, 'endefili');
    
            $this->switchDataSource('default');
        } catch (\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
        
    }
    
    private function saveGeneric ($request, $response, $dataSourceName, $parentTreatment = false){
        $this->entityManager->beginTransaction();
        try {
            $this->switchDataSource($dataSourceName);
            $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
			$this->crudService->save($dataSourceName, $rows);
            
            $this->entityManager->commit(); 
        } catch (\Exception $e){
            if ($parentTreatment) {
                throw $e; 
            }
            $this->entityManager->rollback();
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }
    
    private function switchDataSource($label) {
        $this->dataSourceName = $this->dataSourceNames[$label];
    }
    
    public function setEntityManager($entityManager) {
        $this->entityManager = $entityManager;
    }

    public function getEntityManager() {
        return $this->entityManager;
    }
}
