<?php
namespace Service;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Util\Functions;


class UnidadeService {
    protected $factory;

    /**
     * Construtor do Service.
     *
     * @param Ger01700UnidadeFactory $factory
     */
    public function __construct($factory) {
        $this->factory =  $factory;
    }

    /**
     * Função que cria campos conforme necessário para a
     * inserção/edição de Unidade.
     *
     * @param array $row
     *
     * @return array
     */
    public function validateUnidadeRow($row) {
        if(!Functions::arrayKeyExists('NMRAZSOCFILI', $row) || !$row['NMRAZSOCFILI'])
            $row['NMRAZSOCFILI'] = $row['NMFILIAL'];

        if(!Functions::arrayKeyExists('CDALMOXARIFE', $row))
            $row['CDALMOXARIFE'] = ' ';
        if(!Functions::arrayKeyExists('DSALMOXARIFE', $row))
            $row['DSALMOXARIFE'] = ' ';

        if(!Functions::arrayKeyExists('CDLOCALESTOQ', $row))
            $row['CDLOCALESTOQ'] = ' ';
        if(!Functions::arrayKeyExists('DSLOCALESTOQ', $row))
            $row['DSLOCALESTOQ'] = ' ';
            
        if(isset($row["IDFUNCAOFILI"]) && $row["IDFUNCAOFILI"] == 'U')
            $row["IDFUNCAOFILI"] = null;
        
        if(isset($row["IDFUNCAOFILI"]) && $row["IDFUNCAOFILI"] == 'F') {
            $existeFilial = $this->factory->validaExisteFilialFranqueadora($row["CDFILIAL"]);
            if($existeFilial) {
                $mensagemErro = "Não foi possivel salvar a unidade como franqueadora. Unidade: ". $existeFilial["CDFILIAL"]
                                . " " . $existeFilial["NMFILIAL"] . " foi informada como franqueadora !";
                                
                throw new \Exception($mensagemErro);
            }
        }
        $row["IDFUNCAOFILI_OLD"] = $row["IDFUNCAOFILI_OLD"]?? $row["IDFUNCAOFILI"];
        if(isset($row["IDFUNCAOFILI"]) && $row["IDFUNCAOFILI_OLD"] == 'O') {
            $hasParam = false;
            if($row["IDFUNCAOFILI"] <> $row["IDFUNCAOFILI_OLD"]){
                $hasParam = $this->factory->getHasOPLParams($row["CDFILIAL"]);
                if($hasParam)        
                    throw new \Exception("Esta filial possui parametrização como operador logistico, por isso não pode ter sua função alterada");
            }
            if(isset($row["CDFORNOPELOG"]) && $row["CDFORNOPELOG"] <> $row["CDFORNOPELOG_OLD"]){
                $utparamOL = $this->factory->getParamOPL($row["CDFILIAL"]);
                if($utparamOL == 'N'){
                    $hasParam = $hasParam? $hasParam : $this->factory->getHasOPLParams($row["CDFILIAL"]);
                    if($hasParam)
                        throw new \Exception("Esta filial possui parametrização cadastrada como operador logistico, por isso não pode ter seu fornecedor alterado.");
                }
            }
        }
        
        return($row);
    }
    
    /**
     * Função que cria campos conforme necessário para a
     * inserção de Unidade.
     *
     * @param array $row
     *
     * @return array
     */
    public function validateUnidadeNewRow($row) {
        try{
            if(!Functions::arrayKeyExists('IDALMOXINVAENTR', $row))
                $row['IDALMOXINVAENTR'] = 'N';
    
            if(!Functions::arrayKeyExists('IDCTRLESTQ', $row))
                $row['IDCTRLESTQ'] = 'N';
    
            return($row);
        } catch (\Exception $e){
            
        }
    }
    
    /**
     * Função que cria campos conforme necessário para a
     * inserção/edição de Endefili.
     *
     * @param array $row
     *
     * @return array
     */
    public function validateEndefiliRow($row) {
        #concats address description
        if(Functions::arrayKeyExists('DSENDEFILISEP', $row) && Functions::arrayKeyExists('NRENDEFILISEP', $row)){
           $endeLen = strlen($row['DSENDEFILISEP']);
           $numLen = strlen($row['NRENDEFILISEP']);
           if($endeLen + $numLen > 40){
               $row['DSENDEFILISEP'] = substr($row['DSENDEFILISEP'], 0, 39 - $numLen);
           }
        }
        
        $row['DSENDEFILI'] = ((Functions::arrayKeyExists('DSENDEFILISEP', $row) && Functions::arrayKeyExists('NRENDEFILISEP', $row))? $row['DSENDEFILISEP'].', '.$row['NRENDEFILISEP']: '');

        #concats numbers
        $row['NRFAXFILI'] = ((Functions::arrayKeyExists('NRDDDFAXFILI', $row) && Functions::arrayKeyExists('NRFAXFILISEP', $row))? $row['NRDDDFAXFILI'].$row['NRFAXFILISEP']: '');
        $row['NRTELEFILI'] = ((Functions::arrayKeyExists('NRDDDTELEFILI', $row) && Functions::arrayKeyExists('NRTELEFILISEP', $row))? $row['NRDDDTELEFILI'].$row['NRTELEFILISEP']: '');
        $row['NRTELEXFILI'] = ((Functions::arrayKeyExists('NRDDDTELEXFILI', $row) && Functions::arrayKeyExists('NRTELEXFILISEP', $row))? $row['NRDDDTELEXFILI'].$row['NRTELEXFILISEP']: '');
        

        
        #reduces neighborhood name size, if needed
        $row['NMBAIRFILI'] = (Functions::arrayKeyExists('NMBAIRFILI', $row))? substr($row['NMBAIRFILI'], 0, 30) : null;

        return($row);
    }
    
    public function encrypt($stString, $inOffset) {
        $inTamanho = intval(strlen($stString));
        $inMeioAnt = intval($inTamanho/2);
        $inMeioPos = intval($this->bround(($inTamanho/2) + 0.1) + 1);

        $stVetor = $stString;

        for($byAux = 1; $byAux <= $inMeioAnt; $byAux++) {
            $chAux = $stVetor[$byAux-1];
            $stVetor[$byAux-1] = $stVetor[$byAux*2-1];
            $stVetor[$byAux*2-1] = $chAux;
        }
        
        for($byAux = $inTamanho; $byAux >= $inMeioPos; $byAux--) {
            $chAux = $stVetor[$byAux*2 - ($inTamanho + 1)-1];
            $stVetor[$byAux*2 - ($inTamanho + 1)-1] = $stVetor[$byAux-1];
            $stVetor[$byAux-1] = $chAux;
        }

        $stString = '';

        for($byAux = 1; $byAux <= $inTamanho; $byAux++) {
            $stString = $stString.(chr(ord($stVetor[$byAux-1]) + $inOffset));
        }

        return $stString;
    }
    
    function bround($num) {
        if (strpos($num, '.') === false)
            return $num;

        $num = $num;
        $num_arr = explode('.', $num);
        if (empty($num_arr[1])) return $num;
        if($num_arr[1] == 5) {
            if($num_arr[0] % 2) return $num_arr[0]+1;
            else return $num_arr[0];
        } else return round($num);
    }
}